% decomposition framework for startup, young, mature
clear all
close all
% read in growth rates from Stata export by industry category
years = 1987:2013;

% MORE DETAILED INDUSTRY GROUPS
% 4 industry categories:
%   1-manufacturing
%   2-retail trade
%   3-services
%   4-other

sectors = [10,15,20,40,50,52,60,70];
legend_locations = {'SouthWest', 'NorthWest', 'NorthWest', 'SouthWest'}
% sectors = [1];
num_sectors = length(sectors);

% collect employment and transition matrices for each of the sectors
E_cell = cell(num_sectors,1);
E2_cell = cell(num_sectors,1);
E3_cell = cell(num_sectors,1);
P_t_cell = cell(num_sectors,1);  % want cell arrays to pass to blkdiag easily
Pbar_cell = cell(num_sectors,1);


for i=1:num_sectors
sic_str=num2str(sectors(i));
file=strcat('sector_model/bds_2group_wide_sic',sic_str,'_appendix.txt');
figs=strcat('figures/sector_model/',sic_str);

firm_data = dlmread(file, '', 1, 0);
T = size(firm_data, 1);
assert(T == length(years));

% year2           float  %8.0g                  
% emp_s           float  %9.0g                  
% emp_y           float  %9.0g                  
% emp_m           float  %9.0g                  
% emp_t           float  %9.0g                  
% emp_tm1         float  %9.0g                  
% empshr_s        float  %9.0g                  
% empshr_y        float  %9.0g                  
% empshr_m        float  %9.0g                  
% njcr_s          float  %9.0g                  
% njcr_y          float  %9.0g                  
% njcr_m          float  %9.0g                  
% njcrtot         float  %9.0g                  
% firmsurv_y      float  %9.0g                  
% firmsurv_m      float  %9.0g                  
% avgsize_s       float  %9.0g                  
% avgsize_y       float  %9.0g                  
% avgsize_m       float  %9.0g                  
% avgsizegrowth_y float  %9.0g                  
% avgsizegrowth_m float  %9.0g                  
% emplag1shr_s    float  %9.0g                  
% emplag1shr_y    float  %9.0g                  
% emplag1shr_m    float  %9.0g                  
% mu              float  %9.0g                  
% gbar_y          float  %9.0g                  
% gbar_m          float  %9.0g                  
% gc_s            float  %9.0g                  
% gc_y            float  %9.0g                  
% gc_m            float  %9.0g                  
% resid_s         float  %9.0g                  
% resid_y         float  %9.0g                  
% resid_m         float  %9.0g                  
% beta_s          float  %9.0g                  
% beta_y          float  %9.0g                  
% beta_m          float  %9.0g                  
% shock           float  %9.0g         
% estab           long   %12.0g            
% se              float  %9.0g   

% levels
S_t = firm_data(:,2)';
Ey_t = firm_data(:,3)';
Em_t = firm_data(:,4)';
E_t = S_t + Ey_t + Em_t;
E_tm1 = firm_data(:,6)'; % lagged

% average startups for counterfactuals
S_avg8084= firm_data(:,37)'; % mean(S_1980,...,S_1984); do not use
% S_avg8084(1)=S_t(1);
S_avg_secshr= firm_data(:,38)'; % mean(E_1980^sec/E_1980,...,E_1984^sec/E_1984) * S_t
% S_avg_secshr(1)=S_t(1);
S_avg_s_secshr= firm_data(:,39)'; % mean(S_1980^sec/S_1980,...,S_1984^sec/S_1984) * S_t
% S_avg_s_secshr(1)=S_t(1);

% survival 
xy_t = firm_data(:,14)';
xm_t = firm_data(:,15)';

% conditional growth
ny_t = firm_data(:,19)';
nm_t = firm_data(:,20)';

% unconditional growth (measured from flows)
gs_t = firm_data(:,10)'; % S_t/S_t-1 GROSS(!)
gy_t = firm_data(:,11)';
gm_t = firm_data(:,12)';

% lagged incumbent shares
ws_t = firm_data(:,21)';
wy_t = firm_data(:,22)';
wm_t = firm_data(:,23)';

% computed aggregate growth 
g_t = gs_t.*ws_t + (1-wm_t).*gy_t + wm_t.*gm_t;

% trends
mu_t      = firm_data(:,24)';
gbar_y  = firm_data(:,25)';
gbar_m  = firm_data(:,26)';

% cyclical pieces
gcs_t    = firm_data(:,27)';
gcy_t    = firm_data(:,28)';
gcm_t    = firm_data(:,29)';

% residuals
resids_t = firm_data(:,30)';
residy_t = firm_data(:,31)';
residm_t = firm_data(:,32)';

% betas
beta_s  = firm_data(:,33)';
beta_y  = firm_data(:,34)';
beta_m  = firm_data(:,35)';

% shocks
shock   = firm_data(:,36)';

% reconstructed age-specific growth rates 
gds_t = gcs_t + mu_t + resids_t;
gdy_t = gcy_t + gbar_y + residy_t;
gdm_t = gcm_t + gbar_m + residm_t;

% reconstructed growth rate
gd_t = (1+gds_t).*ws_t + wy_t.*gdy_t + wm_t.*gdm_t;

% check data
assert (norm(E_t - firm_data(:,5)', inf) < 1e-6);
assert (norm(1-wy_t-wm_t, inf) < 1e-6) % weights should sum to 1
assert (norm(g_t - firm_data(:,13)', inf) < 1e-6) % decomp g_t should match measured
% unconditional growth (imputed from change in stocks)
% after September 2014 BDS release these should now match almost exactly
% since NJC_t = EMP_t - (2*DENOM_t - EMP_t)
assert (norm(S_t(2:end)./S_t(1:end-1) - gs_t(2:end)) < 1e-6)  
assert (norm(xy_t.*(1+ny_t)-1 - gy_t) < 1e-3) % some roundoff error
assert (norm(xm_t.*(1+nm_t)-1 - gm_t) < 1e-3) 
% should match g_t and its components exactly
assert(norm(gd_t - g_t, 'inf')<1e-6)
assert(norm(gds_t - gs_t + 1, 'inf')<1e-6)
assert(norm(gdy_t - gy_t, 'inf')<1e-6)
assert(norm(gdm_t - gm_t, 'inf')<1e-6)

% aggregate stock/flow adjustment factor (see appendix)
kappa = ones(1,T); % disabled
% kappa = [1, E_tm1(2:end)./E_t(1:end-1)];

% compute q_t from sequence of realized E_t
% ** NOTE ** do this separately for young and old since measured 
% E_t-1 != imputed E_t-1 from denom used to determine growth rates
qy_t = [(Ey_t(2:end)./(1+gy_t(2:end))./kappa(2:end) - S_t(1:end-1))./Ey_t(1:end-1), NaN];
qy_t2 = [1-(Em_t(2:end)./(1+gm_t(2:end))./kappa(2:end) - Em_t(1:end-1))./Ey_t(1:end-1), NaN];

% long run averages
gybar = mean(gy_t(2:end));
gmbar = mean(gm_t(2:end));
qybar = mean(qy_t(1:end-1));
qy2bar = mean(qy_t2(1:end-1));
stbar = mean(S_t);
stbar8084= mean(S_avg8084); % do not use

% define alternative sequences of transition matrices {P_t}
P_t = zeros(3,3,T); % exactly match data
P_t2 = zeros(3,3,T); % exact growth rate, but hold q constant
P_z = zeros(3,3,T); % predicted growth rate and hold q constant
Pbar3 = zeros(3,3,T); % averaged growth, varying exit
Pbar4 = zeros(3,3,T); % averaged exit, varying growth
for t=2:T
    P_t(:,:,t) = [0, 1+gy_t(t), 0; 
        0, qy_t(t-1)*(1+gy_t(t)), (1-qy_t2(t-1))*(1+gm_t(t));
        0, 0, 1+gm_t(t)];
    P_t2(:,:,t) = [0, 1+gy_t(t), 0;
        0, qybar*(1+gy_t(t)), (1-qy2bar)*(1+gm_t(t));
        0, 0, 1+gm_t(t)];
    P_z(:,:,t) = [0, 1+gybar+gcy_t(t), 0; 
        0, qybar*(1+gybar+gcy_t(t)), (1-qy2bar)*(1+gmbar+gcm_t(t));
        0, 0, 1+gmbar+gcm_t(t)];
    Pbar3(:,:,t) = [0, mean(1+ny_t)*xy_t(t), 0; 
        0, qy_t(t-1)*mean(1+ny_t)*xy_t(t), (1-qy_t2(t-1))*mean(1+nm_t)*xm_t(t);
        0, 0, mean(1+nm_t)*xm_t(t)];
    Pbar4(:,:,t) = [0,(1+ny_t(t))*mean(xy_t), 0; 
        0, qy_t(t-1)*(1+ny_t(t))*mean(xy_t), (1-qy_t2(t-1))*(1+nm_t(t))*mean(xm_t);
        0, 0,(1+nm_t(t))*mean(xm_t)];
end

% define constant Pbar matrix
Pbar = mean(P_t(:,:,2:end),3); % average of products
Pbar2 = [0, 1+gybar, 0; % product of averages (preferred)
    0, qybar*(1+gybar), (1-qy2bar)*(1+gmbar);
    0, 0, 1+gmbar];

%% 1987 to 2012 solving forward
% 3 x T matrix of employment distributions
EE_t_data = [S_t; Ey_t; Em_t]; 
EE_t_data2= [S_avg8084; Ey_t; Em_t]; % with mean(S_1980,...,S_1984); do not use!!!
EE_t_data3= [S_avg_secshr; Ey_t; Em_t]; % with modified S_t^j = ebar^j*S_t
EE_t_data4= [S_avg_s_secshr; Ey_t; Em_t]; % with modified S_t^j = sbar^j*S_t
EE_t_model_1 = EE_t_data; % exact {P_t}
EE_t_model_1a = EE_t_data; % qbar fixed {P_t2}
EE_t_model_2 = EE_t_data; % Pbar
EE_t_model_3 = EE_t_data; % Pbar2
EE_t_model_4 = EE_t_data; % Pbar2 and kappabar
EE_t_model_5 = EE_t_data; % {P_z} 
EE_t_model_6 = EE_t_data; % {Pbar3}
EE_t_model_7 = EE_t_data; % {Pbar4}
EE_t_model_8 = EE_t_data; % Pbar and stbar
EE_t_model_9 = EE_t_data2; % Pbar2 and St_avg8084
EE_t_model_10 = EE_t_data2; % P_t and St_avg8084
EE_t_model_11 = EE_t_data3; % Pbar2 and St_secshr
EE_t_model_12 = EE_t_data3; % P_t and St_secschr
EE_t_model_13 = EE_t_data4; % Pbar2 and St_s_secshr
EE_t_model_14 = EE_t_data4; % P_t and St_s_secschr

for t=2:T
    EE_t_model_1(:,t) = P_t(:,:,t)'*EE_t_model_1(:,t-1)*kappa(t) + [1;0;0]*EE_t_data(1,t-1)*(1+gds_t(t)); % [1;0;0]*S_t(t); 
    EE_t_model_1a(:,t) = P_t2(:,:,t)'*EE_t_model_1(:,t-1)*kappa(t) + [1;0;0]*S_t(t);
    EE_t_model_2(:,t) = Pbar'*EE_t_model_2(:,t-1)*kappa(t) + [1;0;0]*S_t(t);
    EE_t_model_3(:,t) = Pbar2'*EE_t_model_3(:,t-1)*kappa(t) + [1;0;0]*S_t(t);
    EE_t_model_4(:,t) = Pbar2'*EE_t_model_4(:,t-1)*mean(kappa) + [1;0;0]*S_t(t);
    EE_t_model_5(:,t) = P_z(:,:,t)'*EE_t_model_5(:,t-1)*kappa(t) + [1;0;0]*S_t(t);
    EE_t_model_6(:,t) = Pbar3(:,:,t)'*EE_t_model_6(:,t-1)*kappa(t) + [1;0;0]*S_t(t);
    EE_t_model_7(:,t) = Pbar4(:,:,t)'*EE_t_model_7(:,t-1)*kappa(t) + [1;0;0]*S_t(t);
    EE_t_model_8(:,t) = Pbar'*EE_t_model_8(:,t-1)*kappa(t) + [1;0;0]*stbar;
    EE_t_model_9(:,t) = Pbar2'*EE_t_model_9(:,t-1)*kappa(t) + [1;0;0]*stbar8084;
    EE_t_model_10(:,t) = P_t(:,:,t)'*EE_t_model_10(:,t-1)*kappa(t) + [1;0;0]*stbar8084;
    EE_t_model_11(:,t) = Pbar2'*EE_t_model_11(:,t-1)*kappa(t) + [1;0;0]*S_avg_secshr(t);
    EE_t_model_12(:,t) = P_t(:,:,t)'*EE_t_model_12(:,t-1)*kappa(t) + [1;0;0]*S_avg_secshr(t);
    EE_t_model_13(:,t) = Pbar2'*EE_t_model_13(:,t-1)*kappa(t) + [1;0;0]*S_avg_s_secshr(t);
    EE_t_model_14(:,t) = P_t(:,:,t)'*EE_t_model_14(:,t-1)*kappa(t) + [1;0;0]*S_avg_s_secshr(t);
end

% output=[years; sum(EE_t_model_9,1); sum(EE_t_model_10,1); sum(EE_t_model_3,1); sum(EE_t_model_1,1)]';
% cd(figs)
% xlswrite('pbar_emp_avg8084.xlsx',output);
% output=[years; sum(EE_t_model_11,1); sum(EE_t_model_12,1); sum(EE_t_model_3,1); sum(EE_t_model_1,1)]';
% xlswrite('pbar_emp_avgsecshr.xlsx',output);
% output=[years; sum(EE_t_model_13,1); sum(EE_t_model_14,1); sum(EE_t_model_3,1); sum(EE_t_model_1,1)]';
% xlswrite('pbar_emp_avg_s_secshr.xlsx',output);
% cd ../../../

%% collect by sector
E_cell{i} = EE_t_data; 
E2_cell{i} = EE_t_data4; % data for "cf2" with S_t^j = sbar^j*S_t
E3_cell{i} = EE_t_data3; % data for "cf3" with S_t^j = ebar^j*S_t
P_t_cell{i} = P_t;
Pbar_cell{i} = Pbar2;

%% CF1: counterfactual startup sequence for mu 1987 to 2012
mubar = 0.02;
gds_t_cf1 = gcs_t + mubar + resids_t; 
% % alternative counterfactual startup sequences
% gds_t_cf1(:) = 0 % no startup growth
% gds_t_cf1(:) = -1; % no startups

EE_t_cf1 = zeros(3,T); 
EE_t_cf1(:,1) = [S_t(1);Ey_t(1);Em_t(1)]; % initialize with actual 1987 data
for t=2:T
    EE_t_cf1(:,t) = P_t(:,:,t)'*EE_t_cf1(:,t-1)*kappa(t) + [1;0;0]*EE_t_cf1(1,t-1)*(1+gds_t_cf1(t));
end
e_t_model_cf1 = bsxfun(@rdivide,EE_t_cf1,sum(EE_t_cf1,1));

% counterfactual employment weights
ws_t_cf1 = [NaN, e_t_model_cf1(1,1:end-1)];
% the following requires the kappa stock flow adjustment to hold exactly
% wm_t_cf1 = [NaN, ((1-qy_t2(1:end-1)).*EE_t_model_cf1(2,1:end-1) + EE_t_model_cf1(3,1:end-1))./sum(EE_t_model_cf1(:,1:end-1), 1)];
% the following is robust to kappa = 1
wm_t_cf1 = [NaN, EE_t_cf1(3,2:end)./(EE_t_cf1(3,2:end) + EE_t_cf1(2,2:end).*(1+gm_t(2:end))./(1+gy_t(2:end)))];

% make sure counterfactul within age group growth rates match data
assert(norm(EE_t_cf1(2,2:end)./kappa(2:end)./(qy_t(1:end-1).*EE_t_cf1(2,1:end-1) + EE_t_cf1(1,1:end-1)) - 1 - gy_t(2:end), inf) < 1e-6)
assert(norm(EE_t_cf1(3,2:end)./kappa(2:end)./((1-qy_t2(1:end-1)).*EE_t_cf1(2,1:end-1) + EE_t_cf1(3,1:end-1)) - 1 - gm_t(2:end), inf) < 1e-6)

% output for cumulative recoveries:
% cd(figs)
% output = 'cf_emp_87-13.xlsx';
% output_emp = [1987:2013;sum(EE_t_cf1)]';
% xlswrite(output,output_emp)
% cd ../../../

end

%% EVOLUTION OF JOINT DISTRIBUTION SECTOR X AGE

H = kron(eye(num_sectors),[1,1,1]); % to aggregate by sector

% get total employment
EEE_t = cell2mat(E_cell);
EEE2_t = cell2mat(E2_cell);
EEE3_t = cell2mat(E3_cell);

% get startup employment by sector (J x T)
SSS_t = EEE_t(1:3:end,:);
bigS_t = sum(SSS_t);

% sector startup employment share s_t^j = S_t^j/S_t (J x T)
sss_t = bsxfun(@rdivide,SSS_t,bigS_t);
sss2_t = bsxfun(@rdivide,EEE2_t(1:3:end,:),bigS_t); % for "cf2" S_t^j was constructed with sbar
sss3_t = bsxfun(@rdivide,EEE3_t(1:3:end,:),bigS_t); % for "cf3" S_t^j was constructed with ebar

% build array of block diagonal total transition matrices
PPP_t = zeros(3*num_sectors,3*num_sectors,T);
for i=1:T
    tmp = cell(num_sectors,1);
    for ii=1:num_sectors
        tmp{ii} = P_t_cell{ii}(:,:,i);
    end
    PPP_t(:,:,i) = blkdiag(tmp{:});
end
PPPbar_t = blkdiag(Pbar_cell{:});
%% CHECK THAT WE CAN MATCH DATA USING ACTUAL LOM
EEE_t_model = zeros(size(EEE_t));
EEE_t_model(:,1) = EEE_t(:,1); % initialize w/ data in 1987
EEE_t_model1b = EEE_t_model;

for i=2:T
    EEE_t_model(:,i) = PPP_t(:,:,i)'*EEE_t_model(:,i-1) + kron(SSS_t(:,i),[1,0,0]');
    EEE_t_model1b(:,i) = PPP_t(:,:,i)'*EEE_t_model1b(:,i-1) + bigS_t(i)*kron(sss_t(:,i),[1,0,0]');
end
norm(EEE_t_model-EEE_t)
norm(EEE_t_model1b-EEE_t)
norm(EEE_t_model1b-EEE_t_model)

%% new "CF2 and CF3"
EEE_t_model1c = EEE_t_model; % Pbar version of data
EEE_t_model2 = EEE_t_model;
EEE_t_model3 = EEE_t_model;
for i=2:T
%     EEE_t_model2(:,i) = PPP_t(:,:,i)'*EEE_t_model2(:,i-1) + bigS_t(i)*kron(sss2_t(:,i),[1,0,0]');
%     EEE_t_model3(:,i) = PPP_t(:,:,i)'*EEE_t_model3(:,i-1) + bigS_t(i)*kron(sss3_t(:,i),[1,0,0]');
    EEE_t_model1c(:,i) = PPPbar_t'*EEE_t_model1c(:,i-1) + bigS_t(i)*kron(sss_t(:,i),[1,0,0]');
    EEE_t_model2(:,i) = PPPbar_t'*EEE_t_model2(:,i-1) + bigS_t(i)*kron(sss2_t(:,i),[1,0,0]');
    EEE_t_model3(:,i) = PPPbar_t'*EEE_t_model3(:,i-1) + bigS_t(i)*kron(sss3_t(:,i),[1,0,0]');

end
%%
%% 2013 ADDED ABOVE, NOT YET FOR BELOW
%% do startup deficit counterfactual

% read aggregate data
firm_data = dlmread('bds_2group_wide.txt', '', 1, 0);
T = size(firm_data, 1);
assert(T == length(years));

bigmu_t      = firm_data(:,24)';
biggcs_t    = firm_data(:,27)';
bigresids_t = firm_data(:,30)';
biggs_t = firm_data(:,10)'; % S_t/S_t-1 GROSS(!)
assert(norm(biggs_t - (1+bigmu_t + biggcs_t + bigresids_t)) < 1e-6);
% S from aggregating over sectors will be slightly different
biggs2_t = [NaN,bigS_t(2:end)./bigS_t(1:end-1)];
eps_t = biggs_t - biggs2_t; % aggregation correction


%%

% counterfactual startup rate
biggs_cf_t = (1 + 0.02 + biggcs_t + bigresids_t - eps_t);

EEE_t_model4 = EEE_t_model;
for i=2:T
    EEE_t_model4(:,i) = PPPbar_t'*EEE_t_model4(:,i-1) + biggs_cf_t(i)*sum(EEE_t_model4(1:3:end,i-1))*kron(sss_t(:,i),[1,0,0]');
end

%% aggregate
% shares by sector; use H to aggegate
ee_t_model1b = H*bsxfun(@rdivide,EEE_t_model1b,sum(EEE_t_model1b));
ee_t_model1c = H*bsxfun(@rdivide,EEE_t_model1c,sum(EEE_t_model1c));
ee_t_model2 = H*bsxfun(@rdivide,EEE_t_model2,sum(EEE_t_model2));
ee_t_model3 = H*bsxfun(@rdivide,EEE_t_model3,sum(EEE_t_model3));
ee_t_model4 = H*bsxfun(@rdivide,EEE_t_model4,sum(EEE_t_model4));

%% table for decomposition
ee_t_model3_norm = bsxfun(@minus,ee_t_model3,ee_t_model3(:,1));
ee_t_model2_norm = bsxfun(@minus,ee_t_model2,ee_t_model2(:,1));
ee_t_model1c_norm = bsxfun(@minus,ee_t_model1c,ee_t_model1c(:,1));
ee_t_model4_norm = bsxfun(@minus,ee_t_model4,ee_t_model4(:,1));

% incumbent lifecycle dynamics explained
ee_t_model3_norm(:,end)./ee_t_model1c_norm(:,end)

% startup shares -- initial
ee_t_model2_norm(:,end)./ee_t_model1c_norm(:,end) - ee_t_model3_norm(:,end)./ee_t_model1c_norm(:,end)

% startup shares -- time-varying
ee_t_model1c_norm(:,end)./ee_t_model1c_norm(:,end) - ee_t_model2_norm(:,end)./ee_t_model1c_norm(:,end)

% overall change
ee_t_model1c_norm(:,end)

% overall change w/0 startup deficit
ee_t_model4_norm(:,end)

%% output table for appendix
headers={'Sector','Inc. LS','SS Init.','SS TV','Overall Ch.','Addl. w/o SD'};
sectors={'Mining';'Constr.';'Manu.';'TCU';'Whl. Trade';'Ret. Trade';'FIRE';'Services'};
xlswrite('table1_appendix.xls',headers)
xlswrite('table1_appendix.xls',sectors,'A2:A9')

output=[ee_t_model3_norm(:,end)./ee_t_model1c_norm(:,end), ...
    ee_t_model2_norm(:,end)./ee_t_model1c_norm(:,end) - ee_t_model3_norm(:,end)./ee_t_model1c_norm(:,end), ...
    ee_t_model1c_norm(:,end)./ee_t_model1c_norm(:,end) - ee_t_model2_norm(:,end)./ee_t_model1c_norm(:,end), ...
    ee_t_model1c_norm(:,end), ee_t_model4_norm(:,end)];
output=100*output;
xlswrite('table1_appendix.xls',output,'B2:F9')
